Imports System.Runtime.InteropServices

Module ListViewStuff
    ' Declare the GetScrollPos API function.
    <DllImport("user32.dll", CharSet:=CharSet.Auto)> _
    Public Function GetScrollPos(ByVal hWnd As Integer, ByVal nBar As Integer) As Integer
    End Function

    Public Const SB_HORZ = 0

    ' Find and column at this position.
    ' Return True if we are successful.
    ' Return False if the mouse is not under a ListView cell.
    Public Function FindListViewRowColumn(ByVal lvw As ListView, ByVal x As Integer, ByVal y As Integer, ByRef row As Integer, ByRef column As Integer) As Boolean
        ' Find the row under the mouse.
        For i As Integer = 0 To lvw.Items.Count - 1
            If lvw.Items(i).Bounds.Contains(x, y) Then
                row = i

                ' See which column is under the mouse.
                column = GetListViewColumn(lvw, x)
                Return True
            End If
        Next i

        Return False
    End Function

    ' Return the column under this X position.
    Public Function GetListViewColumn(ByVal lvw As ListView, ByVal x As Integer) As Integer
        ' Get the horizontal scroll bar's position.
        x += GetScrollPos(lvw.Handle, SB_HORZ)

        ' Get the column headers in their current display order.
        Dim headers As List(Of ColumnHeader) = GetListViewColumnsInDisplayOrder(lvw)

        ' Find the column that includes this X position.
        For i As Integer = 0 To headers.Count - 1
            x -= headers(i).Width
            If x <= 0 Then Return headers(i).Index
        Next i

        Return -1
    End Function

    ' Return the ListView's columns in their display order.
    Public Function GetListViewColumnsInDisplayOrder(ByVal lvw As ListView) As List(Of ColumnHeader)
        Dim headers As New List(Of ColumnHeader)

        ' Find each of the headers in their display order.
        For i As Integer = 0 To lvw.Columns.Count - 1
            ' Find the column with display index i.
            For j As Integer = 0 To lvw.Columns.Count - 1
                If lvw.Columns(j).DisplayIndex = i Then
                    ' This is the one. Add it to the list.
                    headers.Add(lvw.Columns(j))
                    Exit For
                End If
            Next j
        Next i
        Return headers
    End Function
End Module
